import logging
import os
import subprocess

def dbg_check_call(*args, **kwargs):
    logger = logging.getLogger(__name__)
    logger.debug(f"Running: {' '.join(*args)}")
    subprocess.check_call(*args, **kwargs)

def delete_tempfile():
    return os.environ.get('DEB_EDK2_DEBUG', 'false') in ['0', 'false']



#!/usr/bin/env python3

import os
import sys

if __name__ == '__main__':
    with open('./debian/binary-check.remove', 'r') as f:
        removelist = list(map(lambda s: s.strip(), f.readlines()))

    for path in removelist:
        sys.stdout.write("Removing %s\n" % (path))
        os.unlink(path)



from enum import IntEnum
from uuid import UUID

from virt.firmware.varstore import autodetect
from virt.firmware.efi import devpath
from virt.firmware.efi import efivar

class DevicePathType(IntEnum):
    MediaDevicePath = 4

class MediaDevicePathSubType(IntEnum):
    PIWGFirmwareFile = 6
    PIWGFirmwareVolume = 7

FvNameGuid_By_Type = {
    'ArmVirt': "64074afe-340a-4be6-94ba-91b5b4d0f71e",
    'Ovmf': '7cb8bdc9-f8eb-4f34-aaea-3ee4af6516a1',
    'LoongArchVirt': "5d19a5b3-130f-459b-a292-9270a9e6bc62"
}
UefiShellFileGuid = "7c04a583-9e3e-4f1c-ad65-e05268d0b4d1"

def init_shell_entry(varfile_name, varfile_type):
    fv_name_guid = FvNameGuid_By_Type[varfile_type]
    # After booting with an unitialized firmware volume, the shell ends up
    # being assigned to Boot0002. Let's stick with that.
    # Note: Boot0000 always seems to get overwritten to be the
    # BootManagerMenuApp on first boot.
    shell_index = 2

    varstore = autodetect.open_varstore(varfile_name)
    bpath = devpath.DevicePath()
    elem = devpath.DevicePathElem()
    elem.devtype = DevicePathType.MediaDevicePath
    elem.subtype = MediaDevicePathSubType.PIWGFirmwareVolume
    elem.data = UUID(fv_name_guid).bytes_le
    bpath.append(elem)

    elem = devpath.DevicePathElem()
    elem.devtype = DevicePathType.MediaDevicePath
    elem.subtype = MediaDevicePathSubType.PIWGFirmwareFile
    elem.data = UUID(UefiShellFileGuid).bytes_le
    bpath.append(elem)

    varlist = efivar.EfiVarList()
    varlist.set_boot_entry(
        shell_index,
        title="EFI Internal Shell",
        path=bpath,
    )
    varlist.set_boot_next(shell_index)
    varstore.write_varstore(varfile_name, varlist)



#
# Copyright 2019-2022 Canonical Ltd.
# Authors:
# - dann frazier <dann.frazier@canonical.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import shutil
import tempfile
from util import dbg_check_call
from util import delete_tempfile


class FatFsImage:
    def __init__(self, size_in_mb):
        self.f = tempfile.NamedTemporaryFile(
            prefix=__class__.__name__,
            delete=delete_tempfile(),
        )
        self.path = self.f.name

        dbg_check_call(
            [
                'dd', 'if=/dev/zero', 'of=%s' % (self.path),
                'count=0', 'bs=1M', 'seek=%d' % (size_in_mb), 'status=none'
            ]
        )
        new_env = os.environ.copy()
        new_env['PATH'] = f"{os.environ['PATH']}:/sbin"
        dbg_check_call(['mkdosfs', '-F', '32', self.path], env=new_env)

    def mkdir(self, dir):
        dbg_check_call(['mmd', '-i', self.path, dir])

    def makedirs(self, dir):
        dirs = dir.split(os.path.sep)
        for dir_idx in range(1, len(dirs)+1):
            next_dir = os.path.sep.join(dirs[:dir_idx])
            self.mkdir(next_dir)

    def insert_file(self, src, dest):
        dbg_check_call(
            [
                'mcopy', '-i', self.path, src, '::%s' % (dest)
            ]
        )


class EfiBootableIsoImage:
    def __init__(self, eltorito_img):
        with tempfile.TemporaryDirectory() as iso_root:
            eltorito_iso_root = 'boot'
            eltorito_iso_path = os.path.join(eltorito_iso_root, 'efi.img')
            eltorito_local_root = os.path.join(iso_root, eltorito_iso_root)
            eltorito_local_path = os.path.join(iso_root, eltorito_iso_path)

            os.makedirs(eltorito_local_root)
            shutil.copyfile(eltorito_img.path, eltorito_local_path)

            self.f = tempfile.NamedTemporaryFile(
                prefix=__class__.__name__,
                delete=delete_tempfile(),
            )
            self.path = self.f.name

            dbg_check_call(
                [
                    'xorriso', '-as', 'mkisofs', '-J', '-l',
                    '-c', 'boot/boot.cat',
                    '-partition_offset', '16', '-append_partition', '2',
                    '0xef', eltorito_local_path,
                    '-e', '--interval:appended_partition_2:all::',
                    '-no-emul-boot', '-o', self.path, iso_root
                ]
            )


class GrubShellBootableIsoImage(EfiBootableIsoImage):
    def __init__(self, efi_arch, shim_path, grub_path):
        efi_img = FatFsImage(64)
        efi_img.makedirs(os.path.join('EFI', 'BOOT'))
        removable_media_path = os.path.join(
            'EFI', 'BOOT', 'BOOT%s.EFI' % (efi_arch.upper())
        )
        grub_dest = os.path.join(
            'EFI', 'BOOT', 'GRUB%s.EFI' % (efi_arch.upper())
        )
        efi_img.insert_file(shim_path, removable_media_path)
        efi_img.insert_file(grub_path, grub_dest)
        super().__init__(efi_img)
